﻿using System;
using System.Collections.Generic;
using System.Windows.Media.Media3D;
using ProjectorKit;
using ProjectorKit.Elements;
using ProjectorKit.Events;
using ProjectorKit.Filter;

namespace Layout
{
    public class Program
    {
        #region Constants
        private static int LAYOUT_MARGIN_X = 275;
        private static int LAYOUT_MARGIN_Y = 200;
        private static int LAYOUT_SPACE_X = 200;
        private static int LAYOUT_SPACE_Y = 300;
        private const double WIDTH = 153;
        private const double HEIGHT = 198;
        private const double Z = -290;
        private const double DOC_COUNT = 27;
        #endregion

        #region Variables
        private IEnvironment env = new XmlEnvironment();
        private Point3D[] positions = new Point3D[] {
            new Point3D(-200, 1400, Z),
            new Point3D(100, 1400, Z),
            new Point3D(0, 1600, Z),
            new Point3D(-100, 1200, Z),
            new Point3D(1200, 1400, Z),
            new Point3D(2100, 1400, Z), 
            new Point3D(20, 1600, Z),
            new Point3D(100, 1200, Z), 

            new Point3D(100, 2200, Z), 
            new Point3D(0, 2210, Z),
            new Point3D(-100, 2100, Z), 
            new Point3D(1900, 2250, Z), 
            new Point3D(1800, 2220, Z), 
            new Point3D(1750, 2300, Z), 
            new Point3D(1700, 2290, Z), 
            new Point3D(1750, 1540, Z), 
            new Point3D(1850, 1520, Z), 
            new Point3D(1900, 1500, Z), 
            new Point3D(1800, 1650, Z), 
            new Point3D(-850, 1000, Z), 
            new Point3D(-900, 1100, Z), 
            new Point3D(-800, 1000, Z), 
            new Point3D(-850, 1000, Z), 
            new Point3D(-900, 1400, Z), 
            new Point3D(-900, 1500, Z), 
            new Point3D(-950, 2200, Z), 
            new Point3D(-900, 2200, Z), 
            new Point3D(-700, 2100, Z)
        };
        public ActiveDisplay smartboard;
        public List<ImageElement> onBoardImages = new List<ImageElement>();
        #endregion

        static void Main(string[] args)
        {
            new Program();
        }

        public Program()
        {
            // Set controller and create handler to control the pictures
            AbstractDisplay[] projectors =  {
                (AbstractDisplay)env.World.Get("Dell"),
                (AbstractDisplay)env.World.Get("Projector2")
            };
            smartboard = (ActiveDisplay)env.World.Get("Smartboard");

            for (int i = 0; i < projectors.Length && i < WiiRemoteElement.All.Count; i++)
            {
                WiiRemoteElement wre = WiiRemoteElement.All[i];
                new LayoutHandler(wre.ButtonA, this, smartboard);
                wre.AttachedTo(projectors[i]);
                env.CursorDB.Register(wre);
            }

            // Add a bunch of images into the world
            for (int i = 1; i <= DOC_COUNT; i++)
            {
                ImageElement image = new ImageElement(positions[i - 1] + new Vector3D(0,0, i), WIDTH, HEIGHT, @"layout\Categories" + i + ".png");
                env.World.Add(image);
                if (i <= 8)
                    onBoardImages.Add(image);
            }
            Layout();

            foreach (AbstractDisplay projector in projectors)
            {
                //projector.Filter = new OneEuroFilter(1, 0.2);

                OverlappingDisplaysEvent or = new OverlappingDisplaysEvent(smartboard, projector);
                or.OverlappingStarted += OverlappingCallback;
                or.OverlappingChanged += OverlappingCallback;
                or.OverlappingEnded += OverlappingEnd;
            }
        }

        private void OverlappingCallback(object sender, OverlappingDisplayEventArgs e)
        {
            if (e.DisplayArgs1.NumberOfOverlappingPixels >= e.DisplayArgs2.NumberOfOverlappingPixels)
            {
                // Smartboard has more pixels, only show smartboard
                e.Display1.RemoveBlackout(env.World.GetNameOf(e.Display2));
                e.Display2.AddBlackout(env.World.GetNameOf(e.Display1));
            }
            else
            {
                // Projector has more pixels, clip smartboard
                // Calculate the opacity for a fade-in effect
                double opacity = 1;
                double difference = Math.Min(e.DisplayArgs1.NumberOfOverlappingPixels, e.DisplayArgs2.NumberOfOverlappingPixels) /
                    Math.Max(e.DisplayArgs1.NumberOfOverlappingPixels, e.DisplayArgs2.NumberOfOverlappingPixels);
                if (difference > 0.8)
                    opacity = (1 - difference) * 5;

                e.Display2.RemoveBlackout(env.World.GetNameOf(e.Display1));
                e.Display1.AddBlackout(env.World.GetNameOf(e.Display2), opacity);
            }
        }

        private void OverlappingEnd(object sender, OverlappingDisplayEventArgs e)
        {
            // Remove clipping in both viewports
            e.Display1.RemoveBlackout(env.World.GetNameOf(e.Display2));
            e.Display2.RemoveBlackout(env.World.GetNameOf(e.Display1));
        }

        public void Layout()
        {
            // Adjust layout
            int layoutCol = (int)Math.Floor((double)(smartboard.ResolutionX - LAYOUT_MARGIN_X * 2) / LAYOUT_SPACE_X);
            for (int i = 0; i < onBoardImages.Count; i++)
            {
                int col = i % (layoutCol + 1);
                int row = i / (layoutCol + 1);

                onBoardImages[i].PositionOn(smartboard,
                    col * LAYOUT_SPACE_X + LAYOUT_MARGIN_X,
                    row * LAYOUT_SPACE_Y + LAYOUT_MARGIN_Y);
                onBoardImages[i].Picture = onBoardImages[i].Picture.Replace("Categories", "Details");
            }
        }
    }
}
