﻿using System;
using System.Windows.Media.Media3D;
using ProjectorKit;
using ProjectorKit.Elements;
using ProjectorKit.Events;

namespace Map_with_Dynamic_Resolution_in_Overlappings
{
    /// <summary>
    /// Shows a part of the map on the smartboard. Off-screen areas can be revealed with the projector. 
    /// If the projection overlaps the smartboard the overlapping with the lower pixel density is clipped away.
    /// </summary>
    class Program
    {
        private IEnvironment env;

        static void Main(string[] args)
        {
            new Program();
        }

        public Program()
        {
            // Create a new environment
            env = new XmlEnvironment();

            // Add a map image to the world
            env.World.Add(new ImageElement(new Point3D(0, 1400, -60), 3000, 3000, @"images\map.jpeg"));

            // Create OverlappingRelation between the Smartboard and the Projector
            OverlappingDisplaysEvent or = new OverlappingDisplaysEvent(
                (AbstractDisplay)env.World.Get("Smartboard"),
                (AbstractDisplay)env.World.Get("Projector"));
            or.OverlappingStarted += OverlappingCallback;
            or.OverlappingChanged += OverlappingCallback;
            or.OverlappingEnded += OverlappingEnd;
        }

        private void OverlappingCallback(object sender, OverlappingDisplayEventArgs e)
        {
            if (e.DisplayArgs1.NumberOfOverlappingPixels >= e.DisplayArgs2.NumberOfOverlappingPixels)
            {
                // Smartboard has more pixels, only show smartboard
                e.Display1.RemoveBlackout(env.World.GetNameOf(e.Display2));
                e.Display2.AddBlackout(env.World.GetNameOf(e.Display1));
            }
            else
            {
                // Projector has more pixels, clip smartboard
                // Calculate the opacity for a fade-in effect
                double opacity = 1;
                double difference = Math.Min(e.DisplayArgs1.NumberOfOverlappingPixels, e.DisplayArgs2.NumberOfOverlappingPixels) /
                    Math.Max(e.DisplayArgs1.NumberOfOverlappingPixels, e.DisplayArgs2.NumberOfOverlappingPixels);
                if (difference > 0.8)
                    opacity = (1 - difference) * 5;

                e.Display2.RemoveBlackout(env.World.GetNameOf(e.Display1));
                e.Display1.AddBlackout(env.World.GetNameOf(e.Display2), opacity);
            }
        }

        private void OverlappingEnd(object sender, OverlappingDisplayEventArgs e)
        {
            // Remove clipping in both viewports
            e.Display1.RemoveBlackout(env.World.GetNameOf(e.Display2));
            e.Display2.RemoveBlackout(env.World.GetNameOf(e.Display1));
        }
    }
}
