﻿using System.Collections.Generic;
using System.Windows.Media.Media3D;
using ProjectorKit;
using ProjectorKit.Elements;
using ProjectorKit.Events;
using ProjectorKit.Filter;

namespace Project_Private_Pictures_On_Display
{
    /// <summary>
    /// Shows a world map. The projector can project its private pictures on 
    /// it. The picture area on the map is automatically blacked out to 
    /// provide a better view.
    /// </summary>
    class Program
    {
        private IEnvironment env;
        private const double WIDTH = 160;
        private const double HEIGHT = 120;
        private const double Z = -59;
        private List<ImageElement> images = new List<ImageElement>()
        {
            new ImageElement(new Point3D(-400, 1400, Z), WIDTH, HEIGHT, @"images\1.jpeg"),
            new ImageElement(new Point3D(200, 1400, Z), WIDTH, HEIGHT, @"images\2.jpeg"),
            new ImageElement(new Point3D(0, 1600, Z), WIDTH, HEIGHT, @"images\3.jpeg"),
            new ImageElement(new Point3D(-200, 1100, Z), WIDTH, HEIGHT, @"images\4.jpeg")
        };

        static void Main(string[] args)
        {
            new Program();
        }

        public Program()
        {
            env = new XmlEnvironment();
            AbstractDisplay display = (AbstractDisplay)env.World.Get("Smartboard");
            AbstractDisplay projector = (AbstractDisplay)env.World.Get("Projector");
            //projector.Filter = new OneEuroFilter(1.0, 0.1);

            ImageElement background = new ImageElement(new Point3D(0, 1400, -60), 1920, 1200, @"images\map.jpeg");
            env.World.Add("Background", background);
            projector.AddBlackout("Background");


            // Blackout all images on smartboard (Picture name is used as element name)
            // Create relation between projector and image to add them to the world.
            foreach (ImageElement image in images)
            {
                display.AddBlackout(image.Picture);
                SelectionEvent overlapping = new SelectionEvent(image, projector);
                overlapping.OverlappingStarted += ActiveOverlapping;
                overlapping.OverlappingChanged += ActiveOverlapping;
                overlapping.OverlappingEnded += NoOverlapping;
            }
            projector.SetPose(new Point3D(0, 1200, 1000), new Vector3D(0, 0, -1), new Vector3D(0, 1, 0));
        }

        private void ActiveOverlapping(object sender, SelectionEventArgs e)
        {
            ImageElement image = (ImageElement)e.Element;
            if (e.ElementArgs.Percentage > 0.5 && !env.World.Contains(image))
                env.World.Add(image.Picture, image);
        }

        private void NoOverlapping(object sender, SelectionEventArgs e)
        {
            env.World.Remove(e.Element);
        }
    }
}
