﻿using System.Windows.Media.Media3D;
using ProjectorKit;
using ProjectorKit.Elements;

namespace Relative_Virtual_Elements
{
    /// <summary>
    /// Positions four images around the "Square" marker.
    /// Moving the marker changes orientation and position of the images.
    /// </summary>
    class Program
    {
        private readonly double WIDTH = 80;
        private readonly double HEIGHT = 60;
        private readonly Point3D[] POSITIONS = new Point3D[] {
            new Point3D(-100, 0, +100),
            new Point3D(-100, 0, -100),
            new Point3D(+100, 0, -100),
            new Point3D(+100, 0, +100)};

        static void Main(string[] args)
        {
            new Program();
        }

        public Program()
        {
            IEnvironment env = new XmlEnvironment();

            // Create a real element represented by "Square"
            // This element is not part of the world, but bound to virtual objects.
            MarkerElement marker = new MarkerElement();
            env.Tracker.BindToTrackedObject(marker, "Square");

            // Create and add relative elements to the world
            for (int i = 1; i <= 4; i++)
            {
                ImageElement ie = new ImageElement(POSITIONS[i - 1], WIDTH, HEIGHT, @"images\" + i + @".jpeg");
                ie.SetPose(ie.Position, new Vector3D(0, 1, 0), new Vector3D(1, 0, 0));
                ie.SetRelativeTo(marker);
                env.World.Add(ie);
            }
        }
    }
}
