﻿using System.Windows.Media.Media3D;
using ProjectorKit.Elements;
using ProjectorKit.Utilities;
using ProjectorKit.World;

namespace ProjectorKit.Controller
{
    /// <summary>
    /// This Handler moves a VirtualElement.
    /// The element is always moved over the new target.
    /// </summary>
    public class MoveHandler : AbstractHandler
    {
        private Vector3D offset = new Vector3D(0, 0, 0);
        private Vector3D oldDirection;

        public MoveHandler(Button button) : base(button) { }

        protected override bool CanHandleElement(AbstractElement element)
        {
            return element is RelativeElement;
        }

        protected override void HandlePressed(AbstractElement controller, AbstractElement target)
        {
            Point3D hitpoint = (Point3D)target.Intersection(controller);
            offset = target.Position - hitpoint;
            oldDirection = target.Direction;
        }

        protected override void HandleMove(AbstractElement controller, AbstractElement target)
        {
            AbstractElement e = ServerWorld.Instance.PointingTarget(controller, target);
            if (e != null)
            {
                Vector3D dir = (Vector3D.DotProduct(e.Direction, controller.Direction) < 0) ? e.Direction : -e.Direction;

                // Rotate offset
                offset = Vector3DExtensions.SameRotation(offset, oldDirection, dir);
                oldDirection = dir;

                Point3D hitpoint = (Point3D)e.Intersection(controller);
                target.SetPose(hitpoint + offset + dir, dir, e.Up);
            }
        }
    }
}
