﻿using System.Windows.Media.Media3D;
using ProjectorKit.Elements;
using ProjectorKit.Utilities;

namespace ProjectorKit.Controller
{
    /// <summary>
    /// This Handler fixes the relative position and orientation of the element
    /// and therefore moves and reorientates the element with the controller. 
    /// </summary>
    public class OnLeashHandler : AbstractHandler
    {
        private double distance;
        private Vector3D oldUp;
        private Vector3D oldDirection;

        public OnLeashHandler(Button button) : base(button) { }

        protected override bool CanHandleElement(AbstractElement element)
        {
            return element is RelativeElement;
        }

        protected override void HandlePressed(AbstractElement controller, AbstractElement target)
        {
            distance = controller.Position.DistanceTo(target.Position);
            oldDirection = controller.Direction;
            oldUp = controller.Up;
        }

        protected override void HandleMove(AbstractElement controller, AbstractElement target)
        {
            target.SetPose(
                controller.Position + controller.Direction * distance,
                Vector3DExtensions.SameRotation(target.Direction, oldDirection, controller.Direction),
                Vector3DExtensions.SameRotation(target.Up, oldUp, controller.Up));

            oldDirection = controller.Direction;
            oldUp = controller.Up;
        }
    }
}
