﻿using System;
using System.Collections.Generic;
using ProjectorKit.Controller;

namespace ProjectorKit.Elements
{
    public class WiiRemoteElement : AbstractElement
    {
        #region static
        private static List<WiiRemoteElement> elements;

        public static List<WiiRemoteElement> All
        {
            get
            {
                if (elements == null)
                {
                    elements = new List<WiiRemoteElement>();
                    WiimoteLib.WiimoteCollection wc = new WiimoteLib.WiimoteCollection();
                    try
                    {
                        wc.FindAllWiimotes();
                    }
                    catch (Exception) { }

                    int i = 0;
                    foreach (WiimoteLib.Wiimote wm in wc)
                    {
                        i++;
                        wm.SetReportType(WiimoteLib.InputReport.Buttons, true);
                        wm.SetLEDs(i == 0, i == 1, i == 2, i == 3);
                        elements.Add(new WiiRemoteElement(wm));
                        System.Console.WriteLine("WiiRemote " + i + " found!");
                    }
                }
                return elements;
            }
        }
        #endregion
        private AbstractElement attachedTo;
        private Button buttonA;
        private Button buttonB;
        private Button button1;
        private Button button2;
        #region Properties
        public Button ButtonA
        {
            get { return buttonA; }
        }
        public Button ButtonB
        {
            get { return buttonB; }
        }
        public Button Button1
        {
            get { return button1; }
        }
        public Button Button2
        {
            get { return button2; }
        }
        #endregion

        public WiiRemoteElement() { }
        public WiiRemoteElement(WiimoteLib.Wiimote wiimote)
        {
            buttonA = new Button(this);
            buttonB = new Button(this);
            button1 = new Button(this);
            button2 = new Button(this);

            wiimote.WiimoteChanged += WiimoteChanged;
            wiimote.Connect();
        }

        private void UpdateButton(Button button, bool newState)
        {
            if ((button.State == ButtonState.Released) == newState)
                buttonA.State = newState ? ButtonState.Pressed : ButtonState.Released;
        }

        private void WiimoteChanged(object sender, WiimoteLib.WiimoteChangedEventArgs args)
        {
            WiimoteLib.WiimoteState ws = args.WiimoteState;
            UpdateButton(buttonA, ws.ButtonState.A);
            UpdateButton(buttonB, ws.ButtonState.B);
            UpdateButton(button1, ws.ButtonState.One);
            UpdateButton(button2, ws.ButtonState.Two);
        }

        public void AttachedTo(AbstractElement element)
        {
            if (attachedTo != null)
                attachedTo.Updated -= AttachedElementUpdated;
            attachedTo = element;
            if (attachedTo != null)
                attachedTo.Updated += AttachedElementUpdated;
        }

        public void AttachedElementUpdated(AbstractElement element)
        {
            SetPose(element.Position, element.Direction, element.Up);
        }
    }
}
