﻿using System;
using System.Windows;
using System.Windows.Media.Media3D;
using GroupLab.iNetwork;
using ProjectorKit.Utilities;

namespace ProjectorKit.Elements
{
    public class ActiveDisplay : AbstractDisplay, IPositionable2D
    {
        private double RANGE_BEFORE_DISPLAY = 1000;
        private double width;
        #region Properties
        public double Width
        {
            get { return width; }
        }
        public double Height
        {
            get { return width / AspectRatio; }
        }
        public Polygon Corners
        {
            get
            {
                return new Polygon(new Point[] {
                    new Point(-Width / 2, Height / 2),
                    new Point(-Width / 2, -Height / 2),
                    new Point(+Width / 2, -Height / 2),
                    new Point(+Width / 2, Height / 2),
                });
            }
        }
        #endregion

        public ActiveDisplay() : base() { }
        public ActiveDisplay(double width, int resolutionX, int resolutionY)
            : base(resolutionX, resolutionY)
        {
            this.width = width;
        }

        public override Camera Camera
        {
            get
            {
                return new OrthographicCamera(Position - RANGE_BEFORE_DISPLAY * Direction, Direction, Up, width);
            }
        }

        public override Point3D PixelOrigin(int x, int y)
        {
            double halfResX = Math.Floor(ResolutionX / 2.0);
            double halfResY = Math.Floor(ResolutionY / 2.0);

            double pixelWidth = width / ResolutionX;

            return Position - RANGE_BEFORE_DISPLAY * Direction
                + pixelWidth * (x - halfResX) * Right
                - pixelWidth * (y - halfResY) * Up;
        }

        public override Vector3D PixelDirection(int x, int y)
        {
            return Direction;
        }

        public override Point ConvertToPixel(Point3D point)
        {
            Vector3D pointAsVector = point - Position;

            double halfResX = Math.Floor(ResolutionX / 2.0);
            double halfResY = Math.Floor(ResolutionY / 2.0);
            double pixelWidth = width / ResolutionX;

            int x = (int)Math.Round((Vector3D.DotProduct(pointAsVector, Right) / pixelWidth) + halfResX);
            int y = (int)Math.Round((Vector3D.DotProduct(pointAsVector, Up) / pixelWidth) + halfResY);

            return new Point(x, ResolutionY - 1 - y);
        }

        public override Point3D? Intersection(Point3D origin, Vector3D direction)
        {
            return (new Polygon3D(Position, Direction, Corners)).Intersection(origin, direction);
        }

        #region Synchronize
        public override void UpdateElement(Message msg)
        {
            width = msg.GetDoubleField("width");
            base.UpdateElement(msg);
        }

        public override Message ToMessage(string messageName)
        {
            Message msg = base.ToMessage(messageName);
            msg.AddField("width", width);
            return msg;
        }
        #endregion
    }
}
