﻿using System.Windows.Media;
using System.Windows.Media.Media3D;
using GroupLab.iNetwork;
using ProjectorKit.Utilities;

namespace ProjectorKit.Elements
{
    public class SolidColorElement : AbstractElement2D
    {
        private Color color = Colors.Red;
        public Color Color
        {
            get { return color; }
            set
            {
                if (!color.Equals(value))
                {
                    color = value;
                    UpdateNotification();
                }
            }
        }

        public SolidColorElement() : base() { }
        public SolidColorElement(double width, double height, Color color)
            : base(width, height)
        {
            Color = color;
        }
        public SolidColorElement(Point3D position, double width, double height, Color color)
            : base(position, width, height)
        {
            Color = color;
        }

        protected override Material Texture()
        {
            DiffuseMaterial material = new DiffuseMaterial();
            material.Brush = new SolidColorBrush(color);
            return material;
        }

        #region Synchronization
        public override Message ToMessage(string messageName)
        {
            Message msg = base.ToMessage(messageName);
            msg.Add("color", color);
            return msg;
        }

        public override void UpdateElement(Message msg)
        {
            color = msg.GetColor("color");
            base.UpdateElement(msg);
        }
        #endregion
    }
}
