﻿using System;
using System.Windows.Media.Media3D;

namespace ProjectorKit.Filter
{
    /// <summary>
    /// Filters the input with OneEuroFilters.
    /// </summary>
    public class OneEuroFilter : AbstractFilter
    {
        private ProjectorKit.Utilities.OneEuroFilter[] filters =
            new ProjectorKit.Utilities.OneEuroFilter[9];
        private long lastMs;
        private bool first = true;

        public OneEuroFilter(double minCutoff, double beta)
        {
            // Create filters
            for (int i = 0; i < 9; i++)
                filters[i] = new ProjectorKit.Utilities.OneEuroFilter(minCutoff, beta);
        }

        public override void Filter(Point3D position, Vector3D direction, Vector3D up)
        {
            long currentTime = DateTime.Now.Ticks / TimeSpan.TicksPerMillisecond;
            if (first)
                first = false;
            else
            {
                double rate = currentTime - lastMs;
                Point3D filterPosition = new Point3D(
                    filters[0].Filter(position.X, rate),
                    filters[1].Filter(position.Y, rate),
                    filters[2].Filter(position.Z, rate));
                Vector3D filterDirection = new Vector3D(
                    filters[3].Filter(direction.X, rate),
                    filters[4].Filter(direction.Y, rate),
                    filters[5].Filter(direction.Z, rate));
                Vector3D filterUp = new Vector3D(
                    filters[6].Filter(up.X, rate),
                    filters[7].Filter(up.Y, rate),
                    filters[8].Filter(up.Z, rate));
                UpdateFilter(filterPosition, filterDirection, filterUp);
            }
            lastMs = currentTime;
        }
    }
}
