﻿using System;
using ProjectorKit.Elements;

namespace ProjectorKit.Events
{
    public enum InteractionState { Start, Move, End, None }

    public class InteractionEventArgs : EventArgs
    {
        public InteractionState State;
        public DateTime StartTime;
        public TimeSpan Duration;
        public int Count;
    }

    public abstract class AbstractInteraction<T>
    {
        private bool isInUpdate = false;
        private bool lastState = false;
        private int count = 0;
        private DateTime startTime;
        private InteractionState state;

        #region Properties
        protected DateTime StartTime
        {
            get { return startTime; }
        }
        protected InteractionState State
        {
            get { return state; }
            set {
                if (value == InteractionState.Start)
                {
                    startTime = DateTime.Now;
                    count++;
                }
                state = value;
            }
        }
        #endregion

        /// <summary>
        /// Override to add your recognizer. Do not call this method directly.
        /// Use NoRecursiveUpdate to bind it to your element. This avoids infinit loops.
        /// </summary>
        /// <param name="element"></param>
        protected abstract void Update(AbstractElement element);
        protected void NoRecursiveUpdate(AbstractElement element)
        {
            if (!isInUpdate)
            {
                isInUpdate = true;
                Update(element);
                isInUpdate = false;
            }
        }

        /// <summary>
        /// Uses an activity flag to automatically determine the state.
        /// </summary>
        /// <param name="newState">If it is active or not.</param>
        protected void UpdateStateWithActivityFlag(bool newState)
        {
            if (!lastState && newState)
                State = InteractionState.Start;
            if (lastState && newState)
                State = InteractionState.Move;
            if (lastState && !newState)
                State = InteractionState.End;
            if (!lastState && !newState)
                State = InteractionState.None;
            lastState = newState;
        }

        /// <summary>
        /// Adds default event arguments to the given argument.
        /// </summary>
        /// <param name="args">Argument to set the default parameters.</param>
        protected void SetDefaultEventArgs(InteractionEventArgs args)
        {
            args.State = state;
            args.StartTime = startTime;
            args.Count = count;

            if (state == InteractionState.Start)
                args.Duration = TimeSpan.FromSeconds(0);
            else
                args.Duration = DateTime.Now - startTime;
        }
    }
}
