﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Media.Media3D;
using ProjectorKit.Elements;
using ProjectorKit.Utilities;

namespace ProjectorKit.Events
{
    public class DisplayArgs
    {
        public Polygon RelationArea; // Rename to Area
        public List<Point[]> PixelOverlapping;
        public double NumberOfOverlappingPixels;
        public double AveragePixelDensity;
        public double Percentage;

        public DisplayArgs(AbstractDisplay display, Polygon area, Overlap overlapping)
        {
            RelationArea = area;
            PixelOverlapping = ToPixelOverlapping(display, overlapping);
            NumberOfOverlappingPixels = AreaPixelSum(PixelOverlapping);
            Percentage = NumberOfOverlappingPixels / display.TotalPixels;
            AveragePixelDensity = NumberOfOverlappingPixels / area.Area;
        }

        private List<Point[]> ToPixelOverlapping(AbstractDisplay vp, Overlap global)
        {
            List<Point[]> result = new List<Point[]>();
            foreach (Polygon polygon in global.Overlaps)
            {
                Point3D[] points = global.PolygonToAbsolute(polygon);
                result.Add(vp.ConvertToPixels(points));
            }
            return result;
        }

        private double AreaPixelSum(List<Point[]> pixelOverlappings)
        {
            double result = 0;
            foreach (Point[] pixels in pixelOverlappings)
                result += (new Polygon(pixels)).Area;
            return result;
        }
    }

    public class ProjectorArgs : DisplayArgs
    {
        public double distance;
        public Angle rotation;
        public Angle intersectionAngle;
        public double minDensity;
        public double maxDensity;
        public ProjectorArgs(AbstractDisplay display, Polygon area, Overlap overlapping) :
            base(display, area, overlapping){ }
    }

    public class ElementArgs
    {
        public double Percentage;

        public ElementArgs(IPositionable2D element, Overlap overlapping)
        {
            Percentage = overlapping.Area / element.Corners.Area;
        }
    }

    public class OverlappingEventArgs : InteractionEventArgs
    {
        public Overlap Overlapping;

        public OverlappingEventArgs(Overlap overlapping)
        {
            Overlapping = overlapping;
        }
    }

    public abstract class AbstractOverlappingEvent<T> : AbstractInteraction<T>
    {
    }
}
