﻿using System.Windows.Media.Media3D;
using ProjectorKit.Elements;
using ProjectorKit.Utilities;

namespace ProjectorKit.Events
{
    public class CrossingEventArgs : InteractionEventArgs
    {
        public AbstractElement Pointer;
        public AbstractElement CrossedElement;
        public Point3D EntryPoint;
        public Vector3D EntryPointFromCenter;
        public Point3D ExitPoint;
        public Vector3D ExitPointFromCenter;
        public double Distance;
    }

    /// <summary>
    /// A relation that triggers a callback when the pointer cuts the given element.
    /// Cutting is when the pointer enters and leaves an element under a given timespan.
    /// </summary>
    public sealed class CrossingEvent : AbstractInteraction<CrossingEventArgs>
    {
        public delegate void CrossingRelationEventHandler(object sender, CrossingEventArgs e);
        public event CrossingRelationEventHandler Crossed = delegate { };
        private double maxDurationInSeconds;
        private PointingEventArgs startPointingEventArgs;

        public CrossingEvent(AbstractElement pointer, AbstractElement cuttedElement, double maxDurationInSeconds = 1.0)
        {
            this.maxDurationInSeconds = maxDurationInSeconds;
            PointingEvent pr = new PointingEvent(pointer, cuttedElement);
            pr.Pointed += Update;
        }

        protected override void Update(AbstractElement element) { }
        protected void Update(object sender, PointingEventArgs e)
        {
            UpdateStateWithActivityFlag(e.State == InteractionState.Start || e.State == InteractionState.Move);

            if (e.State == InteractionState.Start)
                startPointingEventArgs = e;
            if (e.State == InteractionState.End && e.Duration.Seconds <= maxDurationInSeconds)
                Crossed(this, CreateEventArgs(e));
       }

        private CrossingEventArgs CreateEventArgs(PointingEventArgs e)
        {
            CrossingEventArgs args = new CrossingEventArgs();
            args.Duration = e.Duration;
            args.StartTime = e.StartTime;
            args.Count = e.Count;
            args.State = InteractionState.End;

            args.Pointer = e.Pointer;
            args.CrossedElement = e.Target;
            args.EntryPoint = startPointingEventArgs.Intersection;
            args.EntryPointFromCenter = startPointingEventArgs.IntersectionFromCenter;
            args.ExitPoint = e.Intersection;
            args.ExitPointFromCenter = e.IntersectionFromCenter;
            args.Distance = args.EntryPoint.DistanceTo(args.ExitPoint);
            return args;
        }
    }
}
