﻿using System.Collections.Generic;
using System.Windows.Media.Media3D;
using ProjectorKit.Elements;
using ProjectorKit.Utilities;

namespace ProjectorKit.Events
{
    public class OverlappingElementsEventArgs : OverlappingEventArgs
    {
        public AbstractElement Element1;
        public AbstractElement Element2;
        public ElementArgs ElementArgs1;
        public ElementArgs ElementArgs2;

        public OverlappingElementsEventArgs(Overlap overlapping) : base(overlapping) { }
    }

    /// <summary>
    /// This relation triggers callbacks when two elements overlap each other.
    /// </summary>
    public sealed class OverlappingElementsEvent : AbstractOverlappingEvent<OverlappingDisplayEventArgs>
    {
        public delegate void OverlappingElementsEventHandler(object sender, OverlappingElementsEventArgs e);
        public event OverlappingElementsEventHandler Overlapped = delegate { };
        public event OverlappingElementsEventHandler OverlappingStarted = delegate { };
        public event OverlappingElementsEventHandler OverlappingChanged = delegate { };
        public event OverlappingElementsEventHandler OverlappingEnded = delegate { };
        private AbstractElement2D element1;
        private AbstractElement2D element2;

        #region Constructors
        /// <summary>
        /// Create an OverlappingRelation between two elements.
        /// </summary>
        /// <param name="element1">The first element of the overlapping</param>
        /// <param name="element2">The second element of the overlapping</param>
        public OverlappingElementsEvent(AbstractElement2D element1, AbstractElement2D element2)
        {
            this.element1 = element1;
            this.element2 = element2;

            element1.Updated += NoRecursiveUpdate;
            element2.Updated += NoRecursiveUpdate;
        }
        #endregion

        protected override void Update(AbstractElement element)
        {
            Overlap overlapping = Overlap.Of(element1, element2);

            UpdateStateWithActivityFlag(overlapping != null);
            if (State != InteractionState.None)
            {
                OverlappingElementsEventArgs args = CreateEventArgs(overlapping);

                if (State == InteractionState.Start)
                    OverlappingStarted(this, args);
                if (State == InteractionState.Move)
                    OverlappingChanged(this, args);
                if (State == InteractionState.End)
                    OverlappingEnded(this, args);
                Overlapped(this, args);
            }
        }

        private OverlappingElementsEventArgs CreateEventArgs(Overlap overlapping)
        {
            OverlappingElementsEventArgs args = new OverlappingElementsEventArgs(overlapping);
            SetDefaultEventArgs(args);
            args.Element1 = element1;
            args.Element2 = element2;
            if (overlapping != null)
            {
                args.ElementArgs1 = new ElementArgs(element1, overlapping);
                args.ElementArgs2 = new ElementArgs(element2, overlapping);
            }
            return args;
        }
    }
}
