﻿using ProjectorKit.Elements;
using ProjectorKit.Events;
using ProjectorToolkit.Utilities;
using System.Windows.Media.Media3D;

namespace ProjectorKit.Gestures
{
    public class ShakingEventArgs : InteractionEventArgs
    {
        public double Peak;
        public Vector3D MainAxis;
    }

    public class ShakingGesture : AbstractInteraction<ShakingEventArgs>
    {
        public delegate void ShakingHandler(object sender, ShakingEventArgs e);
        public event ShakingHandler Recognized = delegate { };
        private Point3D oldPosition;
        private MotionAnalyzer motionAnalyzer;
        private double minPeak;

        public ShakingGesture(AbstractElement element, double duration, double minPeak=100)
        {
            oldPosition = element.Position;
            this.minPeak = minPeak;
            motionAnalyzer = new MotionAnalyzer(duration);
            element.Updated += NoRecursiveUpdate;
        }

        protected override void Update(AbstractElement element)
        {
            motionAnalyzer.AddPoint(element.Position);
            AnalyzedData data = motionAnalyzer.Analyze();

            if (data.Peak >= minPeak && data.MergedSamples.Count > 5)
            {
                State = InteractionState.Start;
                Recognized(this, CreateEventArgs(data));
                motionAnalyzer.Reset();
            }
            else
                State = InteractionState.None;
        }

        private ShakingEventArgs CreateEventArgs(AnalyzedData data)
        {
            ShakingEventArgs args = new ShakingEventArgs();
            SetDefaultEventArgs(args);

            args.MainAxis = data.MainAxis;
            args.Peak = data.Peak;
            return args;
        }
    }
}
