﻿using ProjectorKit.Elements;
using ProjectorKit.Events;
using ProjectorKit.Gestures;
using ProjectorKit.Utilities;
using ProjectorToolkit.Utilities;
using System.Windows.Media.Media3D;

namespace ProjectorToolkit.Gestures
{
    public class WipingEventArgs : InteractionEventArgs
    {
        public Vector3D Axis;
        public Vector3D MainAxis;
        public double Distance;
        public int Count;
    }

    public class WipingGesture : AbstractInteraction<WipingEventArgs>
    {
        public delegate void WipingHandler(object sender, WipingEventArgs e);
        public event WipingHandler Recognized = delegate { };
        private Vector3D axis;
        private double duration;
        private double minDistance;
        private Point3D oldPosition;
        private int numberOfWipes;
        private MotionAnalyzer motionAnalyzer;

        public WipingGesture(AbstractElement element, double duration, Vector3D axis, double minDistance=100, int numberOfWipes=3)
        {
            this.axis = axis;
            this.minDistance = minDistance;
            this.numberOfWipes = numberOfWipes;
            motionAnalyzer = new MotionAnalyzer(duration);
            element.Updated += NoRecursiveUpdate;
        }

        protected override void Update(AbstractElement element)
        {
            motionAnalyzer.AddPoint(element.Position);
            AnalyzedData data = motionAnalyzer.Analyze();

            // For a wipe, peak needs to be higher than a treshold and occur at least 3 times
            if (data.PeakFor(axis, Angle.FromDegrees(10)) >= minDistance && data.CountFor(axis, Angle.FromDegrees(10)) >= numberOfWipes)
            {
                State = InteractionState.Start;
                Recognized(this, CreateEventArgs(data));
                motionAnalyzer.Reset();
            }
            else
                State = InteractionState.None;
        }

        private WipingEventArgs CreateEventArgs(AnalyzedData data)
        {
            WipingEventArgs args = new WipingEventArgs();
            SetDefaultEventArgs(args);

            args.Axis = axis;
            args.MainAxis = data.MainAxis;
            args.Distance = data.PeakFor(axis, Angle.FromDegrees(10));
            args.Count = data.CountFor(axis, Angle.FromDegrees(10));
            return args;
        }
    }
}
