﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Net;
using GroupLab.iNetwork;
using GroupLab.iNetwork.Tcp;

namespace ProjectorKit.Network
{
    public class BufferServer
    {
        private Server server;

        public delegate void MessageEventHandler(object sender, Message msg);
        public event MessageEventHandler MessageReceived = delegate { };

        protected virtual void OnConnect(Server server, ConnectionEventArgs args) { }
        protected virtual void OnDisconnect(Server server, ConnectionEventArgs args) { }
        protected virtual void OnMessageReceived(Connection sender, Message msg)
        {
            Debug.Assert(msg != null);

            MessageReceived(sender, msg);
            // Broadcast message
            Broadcast(msg, sender);
        }

        public BufferServer()
        {
            SetupServer(new Server("Server"));
        }
        public BufferServer(IPAddress ip)
        {
            SetupServer(new Server("Server", ip));
        }
        public BufferServer(IPAddress ip, int port)
        {
            SetupServer(new Server("Server", ip, port));
        }

        private void SetupServer(Server server)
        {
            this.server = server;
            server.IsDiscoverable = true;
            server.Connection += new ConnectionEventHandler(OnServerConnection);
            server.Start();
        }

        ~BufferServer()
        {
            ShutDown();
        }

        public void ShutDown()
        {
            if (server != null && server.IsRunning)
                server.Stop();
        }

        public void Broadcast(Message msg)
        {
            Broadcast(msg, new List<Connection>());
        }

        public void Broadcast(Message msg, Connection excluded)
        {
            Broadcast(msg, new List<Connection>() { excluded });
        }

        public void Broadcast(Message msg, List<Connection> excluded)
        {
            try
            {
                server.BroadcastMessage(msg, excluded);
            }
            catch (Exception) { }
        }

        private void OnServerConnection(object sender, ConnectionEventArgs e)
        {
            // Safely remove delegate if it exists
            e.Connection.MessageReceived -= OnMessageReceived;
            if (e.ConnectionEvent == ConnectionEvents.Connect)
            {
                e.Connection.MessageReceived += OnMessageReceived;
                OnConnect((Server)sender, e);
            }
            else
                OnDisconnect((Server)sender, e);
        }
        private void OnMessageReceived(object sender, Message msg)
        {
            OnMessageReceived((Connection)sender, msg);
        }
    }
}
