﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Net;
using GroupLab.iNetwork;
using GroupLab.iNetwork.Tcp;
using ProjectorKit.Elements;
using ProjectorKit.World;

namespace ProjectorKit.Network
{
    public class WorldServer : BufferServer
    {
        protected List<Message> focusMessages = new List<Message>();
        protected List<Connection> newConnection = new List<Connection>();

        public WorldServer() : base() { }
        public WorldServer(IPAddress ip) : base(ip) { }
        public WorldServer(IPAddress ip, int port) : base(ip, port) { }

        protected override void OnConnect(Server server, ConnectionEventArgs args)
        {
            newConnection.Add(args.Connection);
            base.OnConnect(server, args);

            // Synchronize the worlds
            foreach (AbstractElement element in ServerWorld.Instance.Elements)
            {
                Message msg = ServerWorld.Instance.UpdateMessageForElement(element);
                args.Connection.SendMessage(msg);
            }

            // Update the focus
            foreach (Message msg in focusMessages)
                args.Connection.SendMessage(msg);
            newConnection.Remove(args.Connection);
        }

        public void Send(Message msg, List<Connection> excluded)
        {
            Debug.Assert(msg != null);

            // Log all focus updates
            if (msg.Name.Equals(WorldMessageNames.Focus))
                focusMessages.Add(msg);

            try
            {
                // Do not send while in update process
                List<Connection> allExcludes = new List<Connection>(excluded);
                allExcludes.AddRange(newConnection);
                base.Broadcast(msg, allExcludes);
            }
            catch (Exception) { }
        }
    }
}
