﻿using System.Collections.Generic;
using System.Windows.Media.Media3D;
using ProjectorKit.Elements;
using ProximityToolkit;
using ProximityToolkit.Networking;
using ProximityToolkit.Presence;

namespace ProjectorKit.Tracking
{

    public class ProximityTracker : ITracker
    {
        #region ProximityAdapter
        private class ProximityAdapter
        {
            private AbstractElement updatable;
            private Vector3D offset;
            private int refreshEveryXEvents;
            private int events;

            public ProximityAdapter(Vector3D offset, int refreshEveryXEvents = 1)
            {
                this.offset = offset;
                this.refreshEveryXEvents = refreshEveryXEvents;
            }

            public void BindTo(AbstractElement updatable)
            {
                this.updatable = updatable;
            }

            protected Point3D WPFCoordinatePointAdjustment(Vector3 vector)
            {
                return new Point3D(-vector.Z, vector.Y, vector.X);
            }
            protected Vector3D WPFCoordinateVectorAdjustment(Vector3 vector)
            {
                return new Vector3D(-vector.Z, vector.Y, vector.X);
            }

            public void ProximityUpdate(PresenceBase presence)
            {
                TrackedPresence tp = (TrackedPresence)presence;
                events++;

                if (events >= refreshEveryXEvents &&
                    updatable != null &&
                    presence.IsVisible &&
                    presence.ProxemicLocation.Location != null &&
                    tp.GetPointer("Direction").Direction != null &&
                    tp.GetPointer("Up").Direction != null)
                {
                    Vector3D direction = WPFCoordinateVectorAdjustment(tp.GetPointer("Direction").Direction);
                    Vector3D up = WPFCoordinateVectorAdjustment(tp.GetPointer("Up").Direction);
                    Vector3D right = Vector3D.CrossProduct(direction, up);

                    Point3D position =
                        WPFCoordinatePointAdjustment(presence.ProxemicLocation.Location) +
                        offset.X * right + offset.Y * up + offset.Z * direction;

                    updatable.SetPose(position, direction, up);
                    events = 0;
                }
            }
        }
        #endregion

        private ProximitySpace proximitySpace;
        private int refreshEveryXEvents;

        public ProximityTracker(int refreshEveryXEvents = 1)
        {
            ProximityClientConnection pcc = new ProximityClientConnection();
            pcc.Start("127.0.0.1", 888, true);
            proximitySpace = pcc.GetSpace();

            this.refreshEveryXEvents = refreshEveryXEvents;
        }

        public void BindToTrackedObject(AbstractElement element, string name)
        {
            Bind(new TrackerBinding(name, element));
        }

        public void Bind(TrackerBinding binding)
        {
            ProximityAdapter adapter = new ProximityAdapter(binding.PositionOffset, refreshEveryXEvents);
            PresenceBase pb = proximitySpace.GetPresence(binding.TrackerName);
            pb.WaitForEmbodiment(0);
            pb.DataUpdated += new PresenceHandler(adapter.ProximityUpdate);
            adapter.ProximityUpdate(pb);
            adapter.BindTo(binding.Element);
        }
    }
}
