﻿using System.Windows;
using System.Windows.Media.Media3D;

namespace ProjectorKit.Utilities
{
    /// <summary>
    /// A 2D polygon in a plane.
    /// </summary>
    public class Polygon3D
    {
        private Plane plane;
        private Polygon polygon;

        public Polygon3D(Plane plane, Polygon polygon)
        {
            this.plane = plane;
            this.polygon = polygon;
        }
        public Polygon3D(Point3D point, Vector3D normal, Polygon polygon)
            : this(new Plane(point, normal), polygon) { }

        public Point3D[] Points
        {
            get
            {
                Point3D[] points = new Point3D[polygon.Length];
                for (int i = 0; i < points.Length; i++)
                    points[i] = plane.AbsoluteCoordinateFor(polygon[i]);
                return points;
            }
        }

        public Point3D? Intersection(Point3D origin, Vector3D direction)
        {
            Point3D? hit = plane.Intersection(origin, direction);
            if (hit != null)
            {
                Point hitIn2D = plane.RelativeCoordinateFor((Point3D)hit);

                if (polygon.Contains(hitIn2D))
                    return hit;
            }
            return null;
        }
    }
}
